<?php
declare(strict_types=1);

namespace FiloBlu\Core\Plugin\Magento\Sales\Api;

use FiloBlu\Core\Model\FbOrderExtensionAttrRepository;
use FiloBlu\Core\Model\Data\FbOrderExtensionAttrFactory;
use \Magento\Framework\Exception\NoSuchEntityException;
use Magento\Framework\Api\ExtensibleDataObjectConverter;

class OrderRepositoryInterface
{

    public $fbOrderExtensionAttrRepository;
    public $fbOrderExtensionAttrFactory;
    public $extensibleDataObjectConverter;

    public function __construct(
        FbOrderExtensionAttrRepository $fbOrderExtensionAttrRepository,
        FbOrderExtensionAttrFactory $fbOrderExtensionAttrFactory,
        ExtensibleDataObjectConverter $extensibleDataObjectConverter
    ){
        $this->fbOrderExtensionAttrRepository = $fbOrderExtensionAttrRepository;
        $this->fbOrderExtensionAttrFactory = $fbOrderExtensionAttrFactory;
        $this->extensibleDataObjectConverter = $extensibleDataObjectConverter;
    }

    public function afterSave(
        \Magento\Sales\Api\OrderRepositoryInterface $subject,
        $result,
        $entity
    ) {
        $extensionAttributes = $entity->getExtensionAttributes();
        $filobluExtensionAttributes = $extensionAttributes->getFilobluExtensionAttributes();

        if (!$filobluExtensionAttributes){
            return $result;
        }

        //The row into the table is missing, trying to set the data if it's delivered!
        if (!$filobluExtensionAttributes->getFborderextensionattrId()){
            $array_of_data_to_save = $this->getfbOrderExtensionAttrData($filobluExtensionAttributes);
            if (count($array_of_data_to_save)){
                $itemDataModel = $this->getDataModelFromOrderId($result->getEntityId());
                foreach ($array_of_data_to_save as $k => $v) {
                    $itemDataModel->setData($k,$v);
                }
                $filobluExtensionAttributes = $itemDataModel;
            }
        }
        $this->fbOrderExtensionAttrRepository->save($filobluExtensionAttributes);
        //$resultAttributes = $result->getExtensionAttributes();
        //$resultAttributes->setFilobluExtensionAttributes($filobluExtensionAttributes);
        //$result->setExtensionAttributes($resultAttributes);
        return $result;
    }

    public function afterGet(
        \Magento\Sales\Api\OrderRepositoryInterface $subject,
        $result,
        $id
    ) {
        if (!$result->getEntityId()){
            return $result;
        }

        $itemDataModel = $this->getDataModelFromOrderId($result->getEntityId());
        $extensionAttributes = $result->getExtensionAttributes();
        $extensionAttributes->setFilobluExtensionAttributes($itemDataModel);
        $result->setExtensionAttributes($extensionAttributes);
        return $result;
    }

    public function afterGetList(
        \Magento\Sales\Api\OrderRepositoryInterface $subject,
        $result,
        $searchCriteria
    ) {
        foreach ($result->getItems() as &$order) {
            if (!$order->getEntityId()){
                continue;
            }
            $itemDataModel = $this->getDataModelFromOrderId($order->getEntityId());
            $extensionAttributes = $order->getExtensionAttributes();
            $extensionAttributes->setFilobluExtensionAttributes($itemDataModel);
            $order->setExtensionAttributes($extensionAttributes);
        }
        return $result;
    }

    public function getfbOrderExtensionAttrData(\FiloBlu\Core\Api\Data\FbOrderExtensionAttrInterface $fbOrderExtensionAttr){
        $array_of_data =  $this->extensibleDataObjectConverter->toNestedArray(
            $fbOrderExtensionAttr,
            [],
            \FiloBlu\Core\Api\Data\FbOrderExtensionAttrInterface::class
        );
        return array_filter($array_of_data);
    }

    public function getDataModelFromOrderId($order_entity_id){
        try {
            $itemDataModel = $this->fbOrderExtensionAttrRepository->getByOrderId($order_entity_id);
        } catch (NoSuchEntityException $e){
            //continue;
            $itemDataModel = ($this->fbOrderExtensionAttrFactory->create())->setOrderId($order_entity_id);
        }
        return $itemDataModel;
    }
}

